<!DOCTYPE html>
<html>
    <head>
        <meta charset="utf-8">
        <!-- meta -->
        <meta name="description" content="">
        <meta name="author" content="">
        <meta name="viewport" content="width=device-width,initial-scale=1">
        <!-- styles -->
        <link rel="stylesheet" href="https://stackpath.bootstrapcdn.com/bootstrap/4.1.3/css/bootstrap.min.css" integrity="sha384-MCw98/SFnGE8fJT3GXwEOngsV7Zt27NXFoaoApmYm81iuXoPkFOJwJ8ERdknLPMO" crossorigin="anonymous">
        <link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/font-awesome/4.7.0/css/font-awesome.min.css" />
        <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/awesomplete/1.1.3/awesomplete.min.css" />

        <link rel="stylesheet" href="https://unpkg.com/axialjs@latest/css/axial.css">

        {% block head %}{% endblock %}
    </head>
    <body>
        <div class="container-fluid">

            {% block content %}{% endblock %}

        </div>

        {% block scripts %}{% endblock %}
    </body>
</html>







